@@ -1,14 +1,15 @@
 use anyhow::{bail, Context, Result};
+use clap::StructOpt;
 use console::style;
 use ignore::{overrides::OverrideBuilder, WalkBuilder};
 use log::{LevelFilter, *};
+use serde_json::json;
 use std::fs;
 use std::io::{stdin, stdout, Read, Write};
 use std::path::Path;
 use std::sync::atomic::{AtomicI32, Ordering};
 use std::sync::Arc;
 use std::time::Instant;
-use structopt::StructOpt;
 use threadpool::ThreadPool;
 
 use stylua_lib::{format_code, Config, OutputVerification, Range};
@@ -29,6 +30,41 @@ enum FormatResult {
     Diff(Vec<u8>),
 }
 
+fn create_diff(
+    opt: &opt::Opt,
+    original: &str,
+    expected: &str,
+    file_name: &str,
+) -> Result<Option<Vec<u8>>> {
+    match opt.output_format {
+        opt::OutputFormat::Standard => output_diff::output_diff(
+            original,
+            expected,
+            3,
+            &format!("Diff in {}:", file_name),
+            opt.color,
+        ),
+        opt::OutputFormat::Unified => output_diff::output_diff_unified(original, expected),
+        opt::OutputFormat::Json => {
+            output_diff::output_diff_json(original, expected)
+                .map(|mismatches| {
+                    serde_json::to_vec(&json!({
+                        "file": file_name,
+                        "mismatches": mismatches
+                    }))
+                    // Add newline to end
+                    .map(|mut vec| {
+                        vec.push(b'\n');
+                        vec
+                    })
+                    // Covert to anyhow::Error
+                    .map_err(|err| err.into())
+                })
+                .transpose()
+        }
+    }
+}
+
 fn format_file(
     path: &Path,
     config: Config,
@@ -51,12 +87,11 @@ fn format_file(
     );
 
     if opt.check {
-        let diff = output_diff::output_diff(
+        let diff = create_diff(
+            opt,
             &contents,
             &formatted_contents,
-            3,
-            &format!("Diff in {}:", path.display()),
-            opt.color,
+            path.display().to_string().as_str(),
         )
         .context("failed to create diff")?;
 
@@ -84,14 +119,8 @@ fn format_string(
         format_code(&input, config, range, verify_output).context("failed to format from stdin")?;
 
     if opt.check {
-        let diff = output_diff::output_diff(
-            &input,
-            &formatted_contents,
-            3,
-            "Diff from stdin:",
-            opt.color,
-        )
-        .context("failed to create diff")?;
+        let diff = create_diff(opt, &input, &formatted_contents, "stdin")
+            .context("failed to create diff")?;
 
         match diff {
             Some(diff) => Ok(FormatResult::Diff(diff)),
@@ -109,12 +138,17 @@ fn format(opt: opt::Opt) -> Result<i32> {
         bail!("no files provided");
     }
 
+    // Check for incompatible options
+    if !opt.check && !matches!(opt.output_format, opt::OutputFormat::Standard) {
+        bail!("--output-format can only be used when --check is enabled");
+    }
+
     // Load the configuration
     let config = config::load_config(&opt)?;
 
     // Handle any configuration overrides provided by options
     let config = config::load_overrides(config, &opt);
-    debug!("config: {config:#?}");
+    debug!("config: {:#?}", config);
 
     // Create range if provided
     let range = if opt.range_start.is_some() || opt.range_end.is_some() {
@@ -182,7 +216,7 @@ fn format(opt: opt::Opt) -> Result<i32> {
                         match handle.write_all(&output) {
                             Ok(_) => (),
                             Err(err) => {
-                                error!("could not output to stdout: {err:#}")
+                                error!("could not output to stdout: {:#}", err)
                             }
                         };
                     }
@@ -195,11 +229,11 @@ fn format(opt: opt::Opt) -> Result<i32> {
                         let mut handle = stdout.lock();
                         match handle.write_all(&diff) {
                             Ok(_) => (),
-                            Err(err) => error!("{err:#}"),
+                            Err(err) => error!("{:#}", err),
                         }
                     }
                 },
-                Err(err) => error!("{err:#}"),
+                Err(err) => error!("{:#}", err),
             }
         }
     });
@@ -260,11 +294,11 @@ fn format(opt: opt::Opt) -> Result<i32> {
                         std::io::ErrorKind::NotFound => {
                             error!("no file or directory found matching '{:#}'", path.display())
                         }
-                        _ => error!("{error:#}"),
+                        _ => error!("{:#}", error),
                     },
-                    _ => error!("{err:#}"),
+                    _ => error!("{:#}", err),
                 },
-                _ => error!("{error:#}"),
+                _ => error!("{:#}", error),
             },
         }
     }
@@ -282,7 +316,7 @@ fn format(opt: opt::Opt) -> Result<i32> {
 }
 
 fn main() {
-    let opt = opt::Opt::from_args();
+    let opt = opt::Opt::parse();
     let should_use_color = opt.color.should_use_color_stderr();
     let level_filter = if opt.verbose {
         LevelFilter::Debug
@@ -321,7 +355,7 @@ fn main() {
     let exit_code = match format(opt) {
         Ok(code) => code,
         Err(err) => {
-            error!("{err:#}");
+            error!("{:#}", err);
             2
         }
     };
