@@ -6,7 +6,10 @@ use crate::{
 #[cfg(feature = "luau")]
 use full_moon::ast::span::ContainedSpan;
 #[cfg(feature = "luau")]
-use full_moon::ast::types::{IndexedTypeInfo, TypeArgument, TypeDeclaration, TypeInfo};
+use full_moon::ast::types::{
+    GenericDeclarationParameter, GenericParameterInfo, IndexedTypeInfo, TypeArgument,
+    TypeDeclaration, TypeInfo,
+};
 use full_moon::ast::{Block, FunctionBody, Parameter};
 use full_moon::{
     ast::{
@@ -58,7 +61,7 @@ pub fn spans_multiple_lines<T: std::fmt::Display>(item: &T) -> bool {
 
 pub fn can_hang_expression(expression: &Expression) -> bool {
     match expression {
-        Expression::Parentheses { expression, .. } => can_hang_expression(expression),
+        Expression::Parentheses { .. } => true, // Can always hang parentheses if necessary
         Expression::UnaryOperator { expression, .. } => can_hang_expression(expression),
         Expression::BinaryOperator { .. } => true, // If a binop is present, then we can hang the expression
         Expression::Value { value, .. } => match &**value {
@@ -225,6 +228,38 @@ pub fn type_info_trailing_trivia(type_info: &TypeInfo) -> Vec<Token> {
     }
 }
 
+#[cfg(feature = "luau")]
+fn generic_declaration_parameter_trailing_trivia(
+    parameter: &GenericDeclarationParameter,
+) -> Vec<Token> {
+    if let Some(default_type) = parameter.default_type() {
+        type_info_trailing_trivia(default_type)
+    } else {
+        match parameter.parameter() {
+            GenericParameterInfo::Name(token) => token.trailing_trivia().cloned().collect(),
+            GenericParameterInfo::Variadic { ellipse, .. } => {
+                ellipse.trailing_trivia().cloned().collect()
+            }
+            other => panic!("unknown node {:?}", other),
+        }
+    }
+}
+
+#[cfg(feature = "luau")]
+pub fn take_generic_parameter_trailing_comments(
+    parameter: &GenericDeclarationParameter,
+) -> (GenericDeclarationParameter, Vec<Token>) {
+    let trailing_comments = generic_declaration_parameter_trailing_trivia(parameter)
+        .iter()
+        .filter(|x| trivia_is_comment(x))
+        .cloned()
+        .collect();
+    (
+        parameter.update_trailing_trivia(FormatTriviaType::Replace(vec![])),
+        trailing_comments,
+    )
+}
+
 fn var_trailing_trivia(var: &Var) -> Vec<Token> {
     match var {
         Var::Name(token_reference) => token_reference
@@ -1078,6 +1113,15 @@ pub fn table_fields_contains_comments(table_constructor: &TableConstructor) -> b
     })
 }
 
+pub fn table_field_trailing_trivia(field: &Field) -> Vec<Token> {
+    match field {
+        Field::ExpressionKey { value, .. } => get_expression_trailing_trivia(value),
+        Field::NameKey { value, .. } => get_expression_trailing_trivia(value),
+        Field::NoKey(expression) => get_expression_leading_trivia(expression),
+        other => panic!("unknown node {:?}", other),
+    }
+}
+
 // Checks to see whether an expression contains comments inline inside of it
 // This can only happen if the expression is a BinOp
 // We should ignore any comments which are trailing for the whole expression, as they are not inline
